@@ -76,11 +76,11 @@ File::Unpack - An aggressive archive file unpacker, based on mime-types
 
 =head1 VERSION
 
-Version 0.25
+Version 0.26
 
 =cut
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 $ENV{PATH} = '/usr/bin:/bin';
 $ENV{SHELL} = '/bin/sh';
@@ -115,6 +115,9 @@ my @builtin_mime_handlers = (
   # Requires: sharutils
   [ 'text=uuencode',        qr{uu},                [qw(/usr/bin/uudecode -o %(destfile)s %(src)s)] ],
 
+  # Requires: upx
+  [ 'application=upx',	   qr{(?:upx\.exe|upx)},   [qw(/usr/bin/upx -q -q -q -d -o%(destfile)s %(src)s) ] ],
+
   # xml.summary.Mono.Security.Authenticode is twice inside of monodoc-1.0.4.tar.gz/Mono.zip/ -> use -o
   [ 'application=zip',        qr{(?:zip|jar|sar)}, [qw(/usr/bin/unzip -P no_pw -q -o %(src)s)] ],
 
@@ -1666,10 +1669,16 @@ sub mime
 	  $r[0] = "text/$1" if $mime2 =~ m{/(\S+)};
 	}
     }
-  elsif ($mime1 eq 'text/plain' and $r[2] =~ m{(?:PostScript|font)}i)
+  elsif (($mime1 eq 'text/plain' and $r[2] =~ m{(?:PostScript|font)}i)
+	or ($mime1 eq 'application/postscript'))
     {
-      # IPA.pfa
-      # ['text/plain; charset=us-ascii','PostScript Type 1 font text (OmegaSerifIPA 001.000)']
+      # 11.3 says:
+      #  IPA.pfa
+      #  ['text/plain; charset=us-ascii','PostScript Type 1 font text (OmegaSerifIPA 001.000)']
+      # sles11 says:
+      #  IPA.pfa
+      #  ['application/postscript', undef, 'PostScript document text']
+      #
       # mime2 = 'application/x-font-type1'
       # $mime2 = eval { File::MimeInfo::Magic::mimetype($in{file}); };
       $mime2 ||= eval { open my $fd,'<',\$in{buf}; File::MimeInfo::Magic::magic($fd); };
@@ -1847,6 +1856,14 @@ sub mime
       $r[0] = "application/x-suffix-$suf+octet-stream";
     }
 
+  if ($r[0] =~ m{^application/x-(ms-dos-|)executable$})
+    {
+      if (-x '/usr/bin/upx')
+        {
+	  $r[0] .= '+upx' unless run(['/usr/bin/upx', '-q', '-q', '-t', $in{file}]);
+	}
+    }
+
   ${$in{uncomp}} = $uncomp_buf if ref $in{uncomp} eq 'SCALAR';
   $r[3] = [ $mime1, $mime2 ] if $mime1 ne $r[0] or ($mime2 and $mime2 ne $mime1);
 
